@@ -373,7 +373,7 @@ atspi_table_get_column_header (AtspiTable *obj,
 
   g_return_val_if_fail (obj != NULL, NULL);
 
-  reply = _atspi_dbus_call_partial (obj, atspi_interface_table, "GetColumnHeader", error, "i", &d_column);
+  reply = _atspi_dbus_call_partial (obj, atspi_interface_table, "GetColumnHeader", error, "i", d_column);
 
   return _atspi_dbus_return_accessible_from_message (reply);
 }
